--[[
    TimerUpdaterLib
    Copyright (C) 2008 Gael Lalire
    Contact: gael.lalire@laposte.net

    TimerUpdaterLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TimerUpdaterLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TimerUpdaterLib.  If not, see <http://www.gnu.org/licenses/>.
--]]

local timerUpdaterLib = LibStub:NewLibrary("TimerUpdaterLib-1.0", 1);
if not timerUpdaterLib then
  return;
end

-- begin import
local factoryLib = LibStub("FactoryLib-1.0");
local Factory = factoryLib.Factory;
local environmentLib = LibStub("EnvironmentLib-1.0");
-- end import

--[[
  The TimerUpdater class.
--]]

TimerUpdater, TimerUpdaterFactory = Factory.CreateAbstractType();
timerUpdaterLib.TimerUpdater = TimerUpdater;
timerUpdaterLib.TimerUpdaterFactory = TimerUpdaterFactory;

-- take only environment event
function TimerUpdater:Enable(timer)
  for _, eventName in ipairs(self.environmentEventList) do
    environmentLib.environment:RegisterEvent(eventName, self, timer);
  end
end

-- take only environment event
function TimerUpdater:Disable(timer)
  for _, eventName in ipairs(self.environmentEventList) do
    environmentLib.environment:UnregisterEvent(eventName, self);
  end
end

-- take only environment event
function TimerUpdaterFactory:Create(eventList)
  local timerUpdater = self:NewInstance();
  timerUpdater.environmentEventList = eventList or {};
  return timerUpdater;
end


--[[
  The TotemTimerUpdater class.
--]]

local TotemTimerUpdater, TotemTimerUpdaterFactory, totemTimerUpdaterFactory = Factory.CreateType(TimerUpdaterFactory);
timerUpdaterLib.TotemTimerUpdater = TotemTimerUpdater;
timerUpdaterLib.TotemTimerUpdaterFactory = TotemTimerUpdaterFactory;
timerUpdaterLib.totemTimerUpdaterFactory = totemTimerUpdaterFactory;

function TotemTimerUpdater:PLAYER_TOTEM_UPDATE(timer)
  local _, _, _, duration = GetTotemInfo(self.slot);
  if duration<=0 then
    timer:StopTimer();
  end  
end

function TotemTimerUpdater:PLAYER_ENTERING_WORLD(timer)
  timer:StopTimer();
end

function TotemTimerUpdater:UNIT_AURA(timer)
  local totemBuffName = self.effect;
  if totemBuffName ~= nil then
    local count = 0;
    if environmentLib.environment:UnitBuff("player", totemBuffName) ~= nil then
      count = count + 1;
    end
    for i=1,4 do
      if environmentLib.environment:UnitBuff("party"..i, totemBuffName) ~= nil then
        count = count + 1;
      end
    end
    if self.raidEffect then
      for i=1,40 do
        if environmentLib.environment:UnitBuff("raid"..i, totemBuffName) ~= nil then
          count = count + 1;
        end
      end    
    end
    timer:SetCount(count);  
  end
end

function TotemTimerUpdaterFactory:Create(slot)
  local totemTimerUpdater = self:NewInstance({"UNIT_AURA", "PLAYER_ENTERING_WORLD", "PLAYER_TOTEM_UPDATE"});
  totemTimerUpdater.slot = slot;
  return totemTimerUpdater;
end

--[[
  The BuffTimerUpdater class.
--]]

local BuffTimerUpdater, BuffTimerUpdaterFactory, buffTimerUpdaterFactory = Factory.CreateType(TimerUpdaterFactory);
timerUpdaterLib.BuffTimerUpdater = BuffTimerUpdater;
timerUpdaterLib.BuffTimerUpdaterFactory = BuffTimerUpdaterFactory;
timerUpdaterLib.buffTimerUpdaterFactory = buffTimerUpdaterFactory;

function BuffTimerUpdater:UNIT_AURA(timer)
  local buff, _, _, count = environmentLib.environment:UnitBuff("player", self.buffName);
  if not self.found then
    if buff ~= nil then
      self.found = true;
      timer:SetCount(count);  
    end
  else
    if buff == nil then
      timer:StopTimer();
    else
      timer:SetCount(count);    
    end
  end
end

function BuffTimerUpdaterFactory:Create(buffName)
  local buffTimerUpdater = self:NewInstance({"UNIT_AURA"});
  buffTimerUpdater.buffName = buffName;
  buffTimerUpdater.found = false;
  return buffTimerUpdater;
end

--[[
  The FriendBuffTimerUpdater class.
--]]

local FriendBuffTimerUpdater, FriendBuffTimerUpdaterFactory, friendBuffTimerUpdaterFactory = Factory.CreateType(TimerUpdaterFactory);
timerUpdaterLib.FriendBuffTimerUpdater = FriendBuffTimerUpdater;
timerUpdaterLib.FriendBuffTimerUpdaterFactory = FriendBuffTimerUpdaterFactory;
timerUpdaterLib.friendBuffTimerUpdaterFactory = friendBuffTimerUpdaterFactory;

function FriendBuffTimerUpdater:UpdateTimer(timer)
  -- if the old path is not valid, refind path
  if self.targetPath==nil or environmentLib.environment:UnitFullName(self.targetPath)~=self.targetName then
    self.targetPath = environmentLib.environment:FindPath(self.targetName);
  end
  
  -- we lock the target of our earth shield
  if self.targetPath~=nil then
    local earthSieldBuff, _, _, count = environmentLib.environment:UnitBuff(self.targetPath, self.buffName);
    if count=="" then
      count = -1;
    else
      count = tostring(count);
    end
    if not self.found then
      if earthSieldBuff~= nil then
        self.found = true;
        self.timer = timer;
        timer:SetCount(count);      
      end
    else
      -- your target has not still your buff
      if earthSieldBuff == nil then
        timer:StopTimer();
      else
        timer:SetCount(count);      
      end
    end
  else
    timer:SetCount(-1);
  end
end

function FriendBuffTimerUpdater:UNIT_AURA(timer)
  self:UpdateTimer(timer);
end

function FriendBuffTimerUpdater:PLAYER_TARGET_CHANGED(timer)
  self:UpdateTimer(timer);
end

function FriendBuffTimerUpdater:PARTY_MEMBERS_CHANGED(timer)
  self:UpdateTimer(timer);
end


function FriendBuffTimerUpdaterFactory:Create(buffName)
  local buffTimerUpdater = self:NewInstance({"UNIT_AURA", "PLAYER_TARGET_CHANGED", "PARTY_MEMBERS_CHANGED"});
  buffTimerUpdater.buffName = buffName;
  buffTimerUpdater.found = false;
  return buffTimerUpdater;
end

--[[
  The FriendUniqueBuffTimerUpdater class.
--]]

local FriendUniqueBuffTimerUpdater, FriendUniqueBuffTimerUpdaterFactory, friendUniqueBuffTimerUpdaterFactory = Factory.CreateType(FriendBuffTimerUpdaterFactory);
timerUpdaterLib.FriendUniqueBuffTimerUpdater = FriendUniqueBuffTimerUpdater;
timerUpdaterLib.FriendUniqueBuffTimerUpdaterFactory = FriendUniqueBuffTimerUpdaterFactory;
timerUpdaterLib.friendUniqueBuffTimerUpdaterFactory = friendUniqueBuffTimerUpdaterFactory;

function FriendUniqueBuffTimerUpdater:Enable(timer)
  self.action:RegisterEvent("ACTION_PERFORMED", self, timer);
  -- super enable
  FriendBuffTimerUpdater.Enable(self, timer);
end

function FriendUniqueBuffTimerUpdater:Disable(timer)  
  -- super disable
  FriendBuffTimerUpdater.Disable(self, timer);
  self.action:UnregisterEvent("ACTION_PERFORMED", self);
end

function FriendUniqueBuffTimerUpdater:ACTION_PERFORMED(timer)
  timer:StopTimer();
end

function FriendUniqueBuffTimerUpdaterFactory:Create(buffName, action)
  local friendUniqueBuffTimerUpdater = self:NewInstance(buffName);
  friendUniqueBuffTimerUpdater.action = action;
  return friendUniqueBuffTimerUpdater;
end
